#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <OneWire.h>

// OneWire DS18S20, DS18B20, DS1822 Temperature Example
//
// http://www.pjrc.com/teensy/td_libs_OneWire.html
//
// The DallasTemperature library can do all this work for you!
// http://milesburton.com/Dallas_Temperature_Control_Library

OneWire  ds(D4);  // on pin 10 (a 4.7K resistor is necessary)
  byte ambient[8] ={0x28,0xAC,0x26,0x06,0x05,0x00,0x00,0x67};
  byte tank[8] = {0x28,0xFF,0xB2,0x8F,0x64,0x15,0x01,0xB1};
  byte toBooster[8];
  byte fromBooster[8]={0x28,0xFF,0x1F,0x8D,0x64,0x15,0x01,0x69};
  byte inlet[8] = {0x28,0xFF,0x09,0xC9,0x64,0x15,0x03,0xDB};
  byte addr[8];

// Wifi Stuff

const char* ssid = "ssid";
const char* password = "abcd";
const char* mqtt_server = "192.168.1.186";
WiFiClient espClient;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;



void setup(void) {
  Serial.begin(115200);
  client.setServer(mqtt_server, 1883);
  setup_wifi();
  
  
}

void loop(void) {
  
  if (!client.connected()) {
    reconnect();
    }
  delay (1000);
  Dataread();
  
  
  
ESP.deepSleep(15 * 60 * 1000000);
  
  //client.loop();  
}

void setup_wifi() {
// setup Wifi and connevt to AP

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Client")) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      client.publish("test", "hello world Hotwater");
      // ... and resubscribe
      client.subscribe("inTopic");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void Dataread(){
    float k;
    float l;
    
    l = Battery_Read();
    Serial.print("  Battery Voltage         = ");
    Serial.print(l);
    Serial.println(" V ");
    Serial.println("");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/BATTERY", String(l).c_str(), true);
  
    l = DSread(ambient);
    Serial.print("  Ambient Temperature     = ");
    Serial.print(l);
    Serial.println(" Celsius ");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/TEMP/AMBIENT", String(l).c_str(), true);
    
  

    l = DSread(tank);
    Serial.print("  Tank Temperature        = ");
    Serial.print(l);
    Serial.println(" Celsius ");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/TEMP/TANK", String(l).c_str(), true);
  
  
/*
    l = DSread(toBooster);
    Serial.print("  To Booster Temperature  = ");
    Serial.print(l);
    Serial.println(" Celsius ");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/TEMP/TOBOOSTER", String(l).c_str(), true);
  */
  

    l = DSread(fromBooster);
    Serial.print("  From Booster Temperature = ");
    Serial.print(l);
    Serial.println(" Celsius ");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/TEMP/BOOSTER", String(l).c_str(), true);
  

    l = DSread(inlet);
    Serial.print("  Inlet Temperature        = ");
    Serial.print(l);
    Serial.println(" Celsius ");
    Serial.print("Publish message: ");
    Serial.println(l);
    client.publish("HOME/SOLAR/WATER/TEMP/INLET", String(l).c_str(), true);
  
   
}

float DSread(byte addr[]) {
    byte i;
    byte present = 0;
    byte data[12];
    float celsius;
  
 
    ds.reset();
    ds.select(addr);
    ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
    delay(1000);              // Wait for conversion
    present = ds.reset();
    ds.select(addr);    
    ds.write(0xBE);         // Read Scratchpad

    for ( i = 0; i < 9; i++) {           // we need 9 bytes
      data[i] = ds.read();
    }

    // Convert the data to actual temperature
    // because the result is a 16 bit signed integer, it should
    // be stored to an "int16_t" type, which is always 16 bits
    // even when compiled on a 32 bit processor.
      int16_t raw = (data[1] << 8) | data[0];
      byte cfg = (data[4] & 0x60);
      // at lower res, the low bits are undefined, so let's zero them
      if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
      else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
      else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
      // default is 12 bit resolution, 750 ms conversion time

    celsius = (float)raw / 16.0;
    return celsius;
  
}

float Battery_Read(){
  int i;
  float BV = 0;

  for (i = 0; i < 5; i++){
    BV = BV + analogRead(A0);    // sensor on analog pin 0
    //Serial.println (analogRead(A0));
    //Serial.println (BV);
  }

  BV = BV / 5;    // average  
  BV=(BV*0.0127); // esp12e has 1V max on Analog input but Node MCU has voltage divider  to allow 3V on A0 add 1.22Mohm inseries with A0 to allow reading voltages of upto 15.4v
  //Serial.println (BV);
  return BV;
}
 
 
  
  
